
#ifndef VTKFILTERSSTATISTICS_EXPORT_H
#define VTKFILTERSSTATISTICS_EXPORT_H

#ifdef VTKFILTERSSTATISTICS_STATIC_DEFINE
#  define VTKFILTERSSTATISTICS_EXPORT
#  define VTKFILTERSSTATISTICS_NO_EXPORT
#else
#  ifndef VTKFILTERSSTATISTICS_EXPORT
#    ifdef FiltersStatistics_EXPORTS
        /* We are building this library */
#      define VTKFILTERSSTATISTICS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKFILTERSSTATISTICS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKFILTERSSTATISTICS_NO_EXPORT
#    define VTKFILTERSSTATISTICS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKFILTERSSTATISTICS_DEPRECATED
#  define VTKFILTERSSTATISTICS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKFILTERSSTATISTICS_DEPRECATED_EXPORT
#  define VTKFILTERSSTATISTICS_DEPRECATED_EXPORT VTKFILTERSSTATISTICS_EXPORT VTKFILTERSSTATISTICS_DEPRECATED
#endif

#ifndef VTKFILTERSSTATISTICS_DEPRECATED_NO_EXPORT
#  define VTKFILTERSSTATISTICS_DEPRECATED_NO_EXPORT VTKFILTERSSTATISTICS_NO_EXPORT VTKFILTERSSTATISTICS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKFILTERSSTATISTICS_NO_DEPRECATED
#    define VTKFILTERSSTATISTICS_NO_DEPRECATED
#  endif
#endif

#endif /* VTKFILTERSSTATISTICS_EXPORT_H */
